import java.applet.Applet;
import java.net.*;

//---------------------------------------------------
// This class is public domain
// Author: Ugo Chirico
// e-mail: ugos@geocities.com
// Home Page: http:/www.geocities.com/ResearchTriangle/3219
// Version 1.0.0
//---------------------------------------------------
public abstract class LightServer extends Applet
{
	// Attributi del tag <body> della pagina risultante
	// E' possibile impostare un semplice template per la pagina risultante
	// inserendo gli attributi del <body> in uno dei parametri dell'applet
	protected String m_strBodyAttributes;
	
	// Query impostata nella pagina html
	private String m_strQuery;

	// Inizializza l'applet e imposta gli attributi del body	
	public void init()
	{
		super.init();
		m_strBodyAttributes = getParameter("bodyattributes");
	}

	// Questo metodo e' chiamato dal codice JavaScript nella pagina html	
	// per impostare la query. 
	// La sintassi della query e' la seguente: param1=valore1&param2=valore2
	public final String postQuery(String strQuery)
	{
		// Imposta la query aggiungendo il carattere '&' all'inizio ed alla fine per 
		// facilitare l'estrazione delle coppie parametro-valore (vedi getParmValue)
		m_strQuery = "&" + strQuery + "&";

		String strResultPage;
		try
		{
			// Avvia la computazione
			strResultPage = executeQuery();
		}
		catch(LightServerException ex)
		{
			// In caso di errore previsto costruisce la pagina risultante
			strResultPage = ex.getMessagePage();
		}
		catch(Exception ex)
		{
			// In caso di errore imprevisto costruisce la pagina risultante
			strResultPage = getGenericExceptionPage(ex.toString());
		}

		return strResultPage;
	}

	// Questo e' il metodo che deve essere riscritto nella classe derivata
	// per implementare il proprio server
	protected abstract String executeQuery() throws LightServerException;

	// Interpreta la query e restituisce il valore del paramentro richiesto
	protected final String getParmValue(String strParmName) throws LightServerException
	{
		String strSearchString = "&" + strParmName.toUpperCase() + "=";
		
		int nPos = m_strQuery.toUpperCase().indexOf(strSearchString);
		if (nPos == -1)
			return null;

		int nBegin = nPos + strSearchString.length();
		int nEnd = m_strQuery.indexOf("&", nBegin);

		if(nEnd == -1)
		{
			LightServerException lsEx = 
				new LightServerException("Syntax error in parameter:" + strParmName, getBodyAttributes());
			throw lsEx;
		}

		String strParmValue = m_strQuery.substring(nBegin, nEnd);
		return strParmValue;
	}

	// In caso di errore imprevisto costruisce una pagina contenente l'errore
	// Da sovrascrivere per generare una pagina di errore personalizzata	
	protected String getGenericExceptionPage(String strMsg)
	{
		String strPage = 
			"<head><title>Server raise an exception</title></head>\n "
		 	+ "<body " + m_strBodyAttributes + ">\n" 
		 	+ "<hr>\n"
		 	+ "<h2 align='center'><strong>" + "The server raised a generic exception" + "</strong></h2>\n"
		 	+ "<br>\n"
		 	+ "<p align='center'><strong>The message was: " + strMsg + "</strong></p>\n"
		 	+ "<hr>\n"
		 	+ "</body>";

		 return strPage;
	}
	
	// Imposta gli attributi del body
	public final void setBodyAttributes(String strBodyAttributes)
	{
		m_strBodyAttributes = strBodyAttributes;
	}

	// Restituisce gli attributi del body
	public final String getBodyAttributes()
	{
		return m_strBodyAttributes;
	}
}