import java.io.*;
import java.net.*;
import java.util.*;

//---------------------------------------------------
// This class is public domain
// Author: Ugo Chirico
// e-mail: ugos@geocities.com
// Home Page: http://www.geocities.com/ResearchTriangle/3219
// Version 1.0.0
//---------------------------------------------------
public class WebSearchServer extends LightServer
{
	// URL del database
	private URL m_url = null;

	// Stream di input per la lettura del database
	private DataInputStream m_DBStream;

	// Inizializzazione. Legge il valore del parametro dell'applet "database"
	// ed imposta la variabile m_url
	public void init()
	{
		super.init();

		// Legge la URL del database nel parametro "database"
		String strDBFile = getParameter("database");
		try
		{
			m_url = new URL(getCodeBase().toString() + strDBFile);
		}
		catch(MalformedURLException ex)
		{
			System.out.println("MalformedUrlException");
			m_url = null;
		}
	}

	// Implementa l'algoritmo del server
	protected String executeQuery() throws LightServerException
	{
		// Controlla la URL
		if(m_url == null)
		{
			LightServerException lsEx = 
				new LightServerException("Malformed URL", m_strBodyAttributes);
				
			throw lsEx;
		}

		// Apre il database
		try
		{
			m_DBStream = new DataInputStream(m_url.openStream());
		}
		catch (IOException ex)
		{
			LightServerException lsEx = 
				new LightServerException(ex.toString(), m_strBodyAttributes);
				
			throw lsEx;
		}

		// Estrae dalla query il valore del parametro "keyword"
		String strKeywords = getParmValue("keyword");
		if (strKeywords == null)
		{
			// Genera una pagina vuota
			return nullPage();
		}

	// Vettore contenente i risultati della ricerca
		Vector 	resultVect = new Vector();
		
		// Verifica eventuali operatori booleani + per AND e \b per OR
		// e scompone la query
		StringTokenizer tokenizer = new StringTokenizer(strKeywords, "+ ", true);
		Vector andVect = new Vector();
		Vector orVect = new Vector();
		String strOperator = "";
		String strKeyword;

		// Aggiunge l'intera query nella ricerca
		andVect.addElement(strKeywords.toUpperCase());
		orVect.addElement(andVect);
		andVect = new Vector();

		//Separa le keyword inserite
		while(tokenizer.hasMoreTokens())
		{
			strKeyword = (String)tokenizer.nextToken();
			andVect.addElement(strKeyword.toUpperCase());

			if(tokenizer.hasMoreTokens())
			{
				strOperator = (String)tokenizer.nextToken();
				
				if(strOperator.indexOf("+") == -1)
				{
					orVect.addElement(andVect);
					andVect = new Vector();
				}
			}
			else
			{
				orVect.addElement(andVect);
			}
		}

		String 	strLine;
		String 	strRefField;
		String 	strTitleField;
		String 	strDescField;
		String 	strSepField;
		String 	strKeyField;
		boolean bFound = false;
		boolean bAnd;
		int    	nSize = 0;
		int 	i = 0;
		int 	j = 0;
		Vector 	vect;

		try
		{
			while((strLine = m_DBStream.readLine()) != null)
			{
				// Recupera il campo Reference
				strRefField = strLine;
				// Recupera il campo Title
				strTitleField =  m_DBStream.readLine();
				// Recupera il campo Descrizione
				strDescField =  m_DBStream.readLine();
				// Recupera il campo chiave
				strKeyField = "," + m_DBStream.readLine().toUpperCase() + ",";
				// Recupera il separatore
				strSepField = "," + m_DBStream.readLine().toUpperCase() + ",";

				nSize = orVect.size();
				bFound = false;
				bAnd = false;
				strKeyword = "";
				i = 0;
				j = 0;

				while((i < nSize) && !bFound)
				{
					vect = (Vector)orVect.elementAt(i);
					bFound = true;
					j = 0;
					while((bFound) && j < vect.size())
					{
						strKeyword = (String)vect.elementAt(j);
						if(strKeyField.indexOf("," + strKeyword + ",") == -1)
						{
							bFound = false;
						}
						j++;
					}
					i++;
				}

				if(bFound)
				{
					// Inserisce l'elemento trovato nel vettore dei risultati
					String strRef = "<a href=" + strRefField + ">" + strTitleField + "</a>\n"
								+ "<br><small>" + strDescField + "</small>\n";
					resultVect.addElement(strRef);
				}
			}
			
			// Chiude il DB
			m_DBStream.close();
		}
		catch (IOException ex)
		{
			LightServerException lsEx = 
				new LightServerException(ex.toString(), getBodyAttributes());
				
			throw lsEx;
		}

		// L'applet prevede due parametri aggiuntivi "table" e "image" per 
		// specificare le caratteristiche della tabella che ospitera' il database e  
		// l'URL dell'immagine da anteporre ad ogni item risultante dalla ricerca
		String strBKGTABLE = getParameter("table");
		if (strBKGTABLE == null)
		{
			strBKGTABLE = "";
		}
		
		// URL dell'immagine
		String strImage = getParameter("image");
		if (strImage == null)
		{
			strImage = "";
		}

		// Stringa contenente la pagina dei risultati
		String strPage;

		// Costruisce la pagina
		strPage = "<html>"
				 + "<head><title>Search Result</title></head>\n "
				 + "<body " + m_strBodyAttributes + ">\n" 
				 + "<center><table " + strBKGTABLE + "><tr><td>\n"
				 + "<center><h3 align='center'><strong>Search Result</strong></h3>\n"
				 + "<h4 align='center'>Query: " + strKeywords + "</h4></center>\n"
				 + "<hr>\n";
				 
		// Inserisce nella pagina un item per ogni pagina trovata
		if(resultVect.size() > 0)
		{
			for (i = 0; i < resultVect.size(); i++)
			{
				strPage += "<p><img align='middle' src=" + strImage + "><strong>   " 
					    + resultVect.elementAt(i) + "</strong></p>\n";
			} 
		}
		else
		{
			// Se il vettore dei risultati e' vuoto non e' stata trovata nessuna pagina
			// che corrisponde alla query data.
			strPage += 
				"<p><strong>No pages found that match the keywords</strong></p>\n";

		}
		
		// Termina la pagina dei risultati
		strPage += "<hr></td></tr></table></center>\n"
			    + "</body></html>\n";
		
		return strPage;
	}

	private String nullPage()
	{
		return  "<html>"
				+ "<head><title>Search Result</title></head>\n "
				+ "<body " + getBodyAttributes() + ">\n" 
				+ "<h4 align='center'><strong>Search Result</strong></h4>\n"
				+ "<center><table border='2' align='middle' width='90%' bgcolor='#808000'><tr><td>\n"
				+ "<p><strong>No keyword inserted</strong></p>\n"
				+ "<hr>\n"
				+ "</td></tr></table></center>\n"
			 	+ "<hr>\n"
			 	+ "</body></html>\n";
	}
	
	public String getAppletInfo()
	{
		return "WebSearchServer\nBy Ugo Chirico - http://www.geocities.com/ResearchTriangle/3219\nThis Applet is Free";
	}
}